﻿namespace U2UConsult.WCF.Configuration.Sample
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.IO.IsolatedStorage;
    using System.Reflection;
    using System.Windows;
    using System.Windows.Input;
    using U2UConsult.WCF.Configuration.Sample.ServiceReference;

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// The endpoint configuration that will be used.
        /// </summary>
        private string endpointConfiguration =
            @"
                <configuration>
                    <system.serviceModel>
                        <bindings>
                            <basicHttpBinding>
                                <binding name='BasicHttpBinding_IDummyService' closeTimeout='00:01:00'
                                    openTimeout='00:01:00' receiveTimeout='00:10:00' sendTimeout='00:01:00'
                                    allowCookies='false' bypassProxyOnLocal='false' hostNameComparisonMode='StrongWildcard'
                                    maxBufferSize='65536' maxBufferPoolSize='524288' maxReceivedMessageSize='65536'
                                    messageEncoding='Text' textEncoding='utf-8' transferMode='Buffered'
                                    useDefaultWebProxy='true'>
                                    <readerQuotas maxDepth='32' maxStringContentLength='8192' maxArrayLength='16384'
                                        maxBytesPerRead='4096' maxNameTableCharCount='16384' />
                                    <security mode='None'>
                                        <transport clientCredentialType='None' proxyCredentialType='None'
                                            realm='' />
                                        <message clientCredentialType='UserName' algorithmSuite='Default' />
                                    </security>
                                </binding>
                            </basicHttpBinding>
                        </bindings>
                        <client>
                            <endpoint address='http://localhost:2393/DummyService.svc' binding='basicHttpBinding'
                                bindingConfiguration='BasicHttpBinding_IDummyService' contract='ServiceReference.IDummyService'
                                name='IDummyServiceBinding' />
                        </client>
                    </system.serviceModel>
                </configuration>
             ";

        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Calls the WCF service synchronously.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void Call_Synchronous(object sender, MouseButtonEventArgs e)
        {
            this.SyncPanel.Cursor = Cursors.Wait;
            this.SyncResult.Text = string.Empty;

            // Get configuration information.
            string configuration = this.endpointConfiguration;

            // Save configuration and return path.
            string proxyConfigurationPath = this.GetConfigurationFilePath(configuration, "DummyService.config");

            // Call Service
            DummyServiceClient client = new CustomServiceClient(proxyConfigurationPath);

            // Display result
            this.SyncResult.Text = client.HelloWorld();

            // Reset cursor
            this.SyncPanel.Cursor = Cursors.Hand;

            // Cleanup file
            File.Delete(proxyConfigurationPath);
        }

        /// <summary>
        /// Calls the WCF service asynchronously.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void Call_Asynchronous(object sender, MouseButtonEventArgs e)
        {
            this.AsyncPanel.Cursor = Cursors.Wait;
            this.AsyncResult.Text = string.Empty;

            // Get configuration information.
            string configuration = this.endpointConfiguration;

            // Save configuration and return path.
            string proxyConfigurationPath = this.GetConfigurationFilePath(configuration, "DummyService.config");

            // Call Service
            DummyServiceClient client = new CustomServiceClient(proxyConfigurationPath);
            client.HelloWorldCompleted += this.HelloWorldCompleted;
            client.HelloWorldAsync(proxyConfigurationPath);
        }

        /// <summary>
        /// Callback for asynchronous call.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void HelloWorldCompleted(object sender, HelloWorldCompletedEventArgs e)
        {
            // Display result
            this.AsyncResult.Text = e.Result;

            // Reset cursor
            this.AsyncPanel.Cursor = Cursors.Hand;

            // Cleanup file
            File.Delete((string)e.UserState);
        }

        /// <summary>
        /// Saves the configuration information in a file in isolated storage, and returns the path.
        /// </summary>
        /// <param name="configuration">XML configuration elements.</param>
        /// <param name="fileName">Configuration file name</param>
        /// <returns>Full path name of the configuration file.</returns>
        private string GetConfigurationFilePath(string configuration, string fileName)
        {
            // Create a file in isolated storage. 
            IsolatedStorageFile store = IsolatedStorageFile.GetStore(IsolatedStorageScope.User | IsolatedStorageScope.Assembly, null, null);
            IsolatedStorageFileStream stream = new IsolatedStorageFileStream(fileName, FileMode.Create, store);
            StreamWriter writer = new StreamWriter(stream);
            writer.WriteLine(configuration);
            writer.Close();
            stream.Close();

            // Retrieve the actual path of the file (using reflection). 
            string proxyConfigurationPath = stream.GetType().GetField("m_FullPath", BindingFlags.Instance | BindingFlags.NonPublic).GetValue(stream).ToString();

            return proxyConfigurationPath;
        }

        /// <summary>
        /// Closes the window.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void Close_Window(object sender, MouseButtonEventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Starts dragging the window.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void Drag_Window(object sender, MouseButtonEventArgs e)
        {
            try
            {
                this.DragMove();
            }
            catch (Exception)
            {
                // Ignore
            }
        }

        /// <summary>
        /// Promotes my blog.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void Navigate_Blog(object sender, MouseButtonEventArgs e)
        {
            Process.Start("http://blogs.u2u.be/diederik");
        }
    }
}
